@@ -26,7 +26,7 @@ context or provide additional information in their specific WSGI pipeline
 or logging context.
 """
 
-import collections
+import collections.abc
 import functools
 import itertools
 import threading
@@ -49,6 +49,8 @@ _ENVIRON_HEADERS = {
     'project_id': ['HTTP_X_PROJECT_ID',
                    'HTTP_X_TENANT_ID',
                    'HTTP_X_TENANT'],
+    'domain_id': ['HTTP_X_DOMAIN_ID'],
+    'system_scope': ['HTTP_OPENSTACK_SYSTEM_SCOPE'],
     'user_domain_id': ['HTTP_X_USER_DOMAIN_ID'],
     'project_domain_id': ['HTTP_X_PROJECT_DOMAIN_ID'],
     'user_name': ['HTTP_X_USER_NAME'],
@@ -77,7 +79,7 @@ def generate_request_id():
     return 'req-%s' % uuid.uuid4()
 
 
-class _DeprecatedPolicyValues(collections.MutableMapping):
+class _DeprecatedPolicyValues(collections.abc.MutableMapping):
     """A Dictionary that manages current and deprecated policy values.
 
     Anything added to this dictionary after initial creation is considered a
@@ -179,13 +181,12 @@ class RequestContext(object):
     accesses the system, as well as additional request information.
     """
 
-    user_idt_format = u'{user} {tenant} {domain} {user_domain} {p_domain}'
+    user_idt_format = u'{user} {project_id} {domain} {user_domain} {p_domain}'
     # Can be overridden in subclasses to specify extra keys that should be
     # read when constructing a context using from_dict.
     FROM_DICT_EXTRA_KEYS = []
 
     @_renamed_kwarg('user', 'user_id')
-    @_renamed_kwarg('tenant', 'project_id')
     @_renamed_kwarg('domain', 'domain_id')
     @_renamed_kwarg('user_domain', 'user_domain_id')
     @_renamed_kwarg('project_domain', 'project_domain_id')
@@ -219,7 +220,8 @@ class RequestContext(object):
                  service_project_domain_id=None,
                  service_project_domain_name=None,
                  service_roles=None,
-                 global_request_id=None):
+                 global_request_id=None,
+                 system_scope=None):
         """Initialize the RequestContext
 
         :param overwrite: Set to False to ensure that the greenthread local
@@ -228,6 +230,11 @@ class RequestContext(object):
                                  the token as the admin project. Defaults to
                                  True for backwards compatibility.
         :type is_admin_project: bool
+        :param system_scope: The system scope of a token. The value ``all``
+                             represents the entire deployment system. A service
+                             ID represents a specific service within the
+                             deployment system.
+        :type system_scope: string
         """
         # setting to private variables to avoid triggering subclass properties
         self._user_id = user_id
@@ -240,6 +247,7 @@ class RequestContext(object):
         self.user_name = user_name
         self.project_name = project_name
         self.domain_name = domain_name
+        self.system_scope = system_scope
         self.user_domain_name = user_domain_name
         self.project_domain_name = project_domain_name
         self.is_admin = is_admin
@@ -268,9 +276,8 @@ class RequestContext(object):
             self.update_store()
 
     # NOTE(jamielennox): To prevent circular lookups on subclasses that might
-    # point user to user_id we make user/user_id tenant/project_id etc point
+    # point user to user_id we make user/user_id  etc point
     # to the same private variable rather than each other.
-    tenant = _moved_property('project_id', 'tenant', target='_project_id')
     user = _moved_property('user_id', 'user', target='_user_id')
     domain = _moved_property('domain_id', 'domain', target='_domain_id')
     user_domain = _moved_property('user_domain_id',
@@ -309,6 +316,8 @@ class RequestContext(object):
         return _DeprecatedPolicyValues({
             'user_id': self.user_id,
             'user_domain_id': self.user_domain_id,
+            'system_scope': self.system_scope,
+            'domain_id': self.domain_id,
             'project_id': self.project_id,
             'project_domain_id': self.project_domain_id,
             'roles': self.roles,
@@ -323,13 +332,14 @@ class RequestContext(object):
         """Return a dictionary of context attributes."""
         user_idt = self.user_idt_format.format(
             user=self.user_id or '-',
-            tenant=self.project_id or '-',
+            project_id=self.project_id or '-',
             domain=self.domain_id or '-',
             user_domain=self.user_domain_id or '-',
             p_domain=self.project_domain_id or '-')
 
         return {'user': self.user_id,
-                'tenant': self.project_id,
+                'project_id': self.project_id,
+                'system_scope': self.system_scope,
                 'project': self.project_id,
                 'domain': self.domain_id,
                 'user_domain': self.user_domain_id,
@@ -353,6 +363,19 @@ class RequestContext(object):
                   'user_domain_name': self.user_domain_name,
                   'project_domain_name': self.project_domain_name}
         values.update(self.to_dict())
+        if self.auth_token:
+            # NOTE(jaosorior): Gotta obfuscate the token since this dict is
+            # meant for logging and we shouldn't leak it.
+            values['auth_token'] = '***'  # nosec
+        else:
+            values['auth_token'] = None
+        # NOTE(bnemec: auth_token_info isn't defined in oslo.context, but it's
+        # a common pattern in project context subclasses so we handle it here.
+        # It largely contains things that we don't want logged, like the token
+        # itself (which needs to be removed for security) and the catalog
+        # (which needs to be removed because it bloats the logs terribly).
+        values.pop('auth_token_info', None)
+
         return values
 
     @property
@@ -367,7 +390,6 @@ class RequestContext(object):
 
     @classmethod
     @_renamed_kwarg('user', 'user_id')
-    @_renamed_kwarg('tenant', 'project_id')
     @_renamed_kwarg('domain', 'domain_id')
     @_renamed_kwarg('user_domain', 'user_domain_id')
     @_renamed_kwarg('project_domain', 'project_domain_id')
@@ -375,7 +397,7 @@ class RequestContext(object):
         """Construct a context object from a provided dictionary."""
         kwargs.setdefault('auth_token', values.get('auth_token'))
         kwargs.setdefault('user_id', values.get('user'))
-        kwargs.setdefault('project_id', values.get('tenant'))
+        kwargs.setdefault('project_id', values.get('project_id'))
         kwargs.setdefault('domain_id', values.get('domain'))
         kwargs.setdefault('user_domain_id', values.get('user_domain'))
         kwargs.setdefault('project_domain_id', values.get('project_domain'))
@@ -394,13 +416,13 @@ class RequestContext(object):
                           values.get('project_domain_name'))
         kwargs.setdefault('is_admin_project',
                           values.get('is_admin_project', True))
+        kwargs.setdefault('system_scope', values.get('system_scope'))
         for key in cls.FROM_DICT_EXTRA_KEYS:
             kwargs.setdefault(key, values.get(key))
         return cls(**kwargs)
 
     @classmethod
     @_renamed_kwarg('user', 'user_id')
-    @_renamed_kwarg('tenant', 'project_id')
     @_renamed_kwarg('domain', 'domain_id')
     @_renamed_kwarg('user_domain', 'user_domain_id')
     @_renamed_kwarg('project_domain', 'project_domain_id')
